import pathlib
import shelve
import datetime
import time
from email import utils
from jinja2 import Environment, FileSystemLoader, select_autoescape
from distutils.dir_util import copy_tree

from .post import Post
from .page import Page
from .pandoc import Pandoc

class Site():
    def __init__(self, args, conf):
        self.config = conf
        self.post_inputs = sorted(pathlib.Path(self.config['DIR_POSTS']).glob('**/*.md'), key=lambda p: str(p.name))
        if args.drafts:
            drafts = list(pathlib.Path(self.config['DIR_DRAFTS']).glob('**/*.md'))
            self.post_inputs = sorted(drafts + self.post_inputs, key=lambda p: str(p.name))
        self.page_inputs = pathlib.Path(self.config['DIR_PAGES']).glob('**/*.md')
        self.posts = []
        self.pages = []
        self.topics = {}
        self.jinja_env = Environment(
            loader=FileSystemLoader(self.config['DIR_TEMPLATES']),
            autoescape=select_autoescape(["html", "xml"]),
            trim_blocks=True,
            lstrip_blocks=True,
            keep_trailing_newline=True
        )
        self.site_path = pathlib.Path('.') / self.config['DIR_PUBLISH']
        self.post_store = shelve.open(self.config['POST_STORE'])
        if args.target == 'prod':
            self.config['SITE']['base_url'] = self.config['SITE']['absolute_url']
        self.pandoc = Pandoc(self.config)

    def build(self):
        self.copystaticfiles()
        self.read_posts()
        self.build_topics()
        self.write_posts()
        self.write_index_pages()
        self.write_topic_pages()
        self.read_pages()
        self.write_pages()
        self.write_rss_feed()
        self.post_store.close()

    def copystaticfiles(self):
        copy_tree(self.config['DIR_STATIC'], self.config['DIR_PUBLISH'])

    def read_posts(self):
        for post_input in self.post_inputs:
            draft = False
            if str(post_input).startswith(self.config['DIR_DRAFTS']):
                draft = True
            if post_input.name in self.post_store:
                post = self.post_store[post_input.name]
                print("Reading from post store: ", post)
            else:
                post = Post(post_input, draft)
            if not post.is_current():
                post.build(self.pandoc, self.config)
                print("Building post: ", post)
            self.posts.append(post)
        self.post_paginate()
        self.store_posts()
        self.config['SITE']['posts'] = self.posts
        print("Read {} posts.".format(str(len(self.posts))))

    def read_pages(self):
        for page_input in self.page_inputs:
            page = Page(page_input)
            page.build(self.pandoc, self.config)
            self.pages.append(page)
        print("Read {} static pages.".format(str(len(self.pages))))

    def post_paginate(self):
        for i, post in enumerate(self.posts):
            if i > 0:
                prev_post = self.posts[i-1]
                post.prev_post = {
                    'url': prev_post.url,
                    'title': prev_post.title
                }
            if i < len(self.posts) - 1:
                next_post = self.posts[i+1]
                post.next_post = {
                    'url': next_post.url,
                    'title': next_post.title
                }

    def store_posts(self):
        for post in self.posts:
            self.post_store[post.input_path.name] = post

    def build_topics(self):
        for post in self.posts:
            for topic in post.topics:
                if topic not in self.topics:
                    self.topics[topic] = []
                self.topics[topic].append(post)
        self.config['SITE']['topics'] = self.topics

    def write_posts(self):
        print("Writing posts...")
        for post in self.posts:
            post.write(self.jinja_env, self.config['SITE'])

    def write_index_pages(self):
        print("Writing index pages...")
        template = self.jinja_env.get_template("index.html")
        post_queue = self.posts.copy()
        post_queue.reverse()
        paginator = {
            'page': 1,
            'n_pages': 1 + int(len(post_queue) / self.config['POSTS_PER_PAGE']),
            'prev_url': None,
            'next_url': None
        }
        while len(post_queue) > 0:
            posts = post_queue[:self.config['POSTS_PER_PAGE']]
            page = {}
            if paginator['page'] == 1:
                index_path = self.site_path / "index.html"
                page['url'] = "/"
            else:
                index_path = self.site_path / "page{}".format(str(paginator['page'])) / "index.html"
                page['url'] = "/page{}/".format(paginator['page'])

            if paginator['page'] + 1 <= paginator['n_pages']:
                paginator['next_url'] = "/page{}/".format(str(paginator['page'] + 1))
            else:
                paginator['next_url'] = None

            if paginator['page'] == 2:
                paginator['prev_url'] = "/"
            elif paginator['page'] > 2:
                paginator['prev_url'] = "/page{}/".format(str(paginator['page'] - 1))
            else:
                paginator['prev_url'] = None

            template_out = template.render(posts=posts, site=self.config['SITE'], page=page, paginator=paginator)
            index_path.parent.mkdir(parents=True, exist_ok=True)
            index_path.write_text(template_out)
            post_queue = post_queue[self.config['POSTS_PER_PAGE']:]
            paginator['page'] += 1


    def write_topic_pages(self):
        print("Writing topic pages...")
        template = self.jinja_env.get_template("topic.html")
        for topic in self.topics:
            print("  {}".format(topic))
            posts = self.topics[topic]
            posts.reverse()
            page = {
                'title': topic.title(),
                'url': "/{}/{}/".format(self.config['TOPICS_URL_PREFACE'], topic)
            }
            template_out = template.render(topic=topic, posts=posts, site=self.config['SITE'], page=page)
            topic_path = self.site_path / self.config['TOPICS_URL_PREFACE'] / topic / "index.html"
            topic_path.parent.mkdir(parents=True, exist_ok=True)
            topic_path.write_text(template_out)

    def write_pages(self):
        print("Writing static pages...")
        for page in self.pages:
            page.write(self.jinja_env, self.config['SITE'])

    def write_rss_feed(self):
        print("Writing RSS feed...")
        # self.config['SITE']['timestamp'] = utils.format_datetime(datetime.datetime.utcnow())
        self.config['SITE']['timestamp'] = self.posts[len(self.posts)-1].date
        template = self.jinja_env.get_template("feed.xml")
        template_out = template.render(site=self.config['SITE'], page={})
        output_path = self.site_path / self.config['SITE']['rssfeed']
        output_path.write_text(template_out)




import argparse
import pathlib
import importlib.resources

from ._version import __version__
from .config import *
from .init import init
from .clean import clean
from .build import build
from .serve import serve
from .deploy import deploy

import sys

def main():
    conf = read_config()

    parser = argparse.ArgumentParser(description='A simple static site generator with deployment to S3/Cloudfront.')
    parser.add_argument('-v', '--version', action='version', version='%(prog)s ' + __version__)
    subparsers = parser.add_subparsers()

    parser_init = subparsers.add_parser('init')
    parser_init.set_defaults(func=init)

    parser_clean = subparsers.add_parser('clean')
    parser_clean.set_defaults(func=clean)

    parser_build = subparsers.add_parser('build')
    parser_build.add_argument('--target', default='dev', choices=['dev', 'prod'])
    parser_build.add_argument('--drafts', action='store_true')
    parser_build.set_defaults(func=build)

    parser_serve = subparsers.add_parser('serve')
    parser_serve.set_defaults(func=serve)

    parser_deploy = subparsers.add_parser('deploy')
    parser_deploy.add_argument('--dry-run', action='store_true')
    parser_deploy.set_defaults(func=deploy)

    args = parser.parse_args()
    args.func(args, conf)




"""
Tests?  We don't need no stinkin' tests!

Pull requests welcome ;)

Here's the problem with tests.  Life's too short to write them.  My approach to
testing is the following:
1) Run the program.
2) If it breaks, fix it.

How many times have you come across a package where the tests folder contains
more code than the actual project?  Is this really a good way of spending your
precious coding time?  I know this philosophy is not going to win me much
support, and it's quite a dangerous and childish and wholly unprofessional way
of developing software.  Maybe I'll fix this in the future, but for now I really
really don't have time to write tests for this package and I'm not going to
stress about it, no matter how much grief I get for it.

"""



from .basepage import BasePage

class Page(BasePage):
    def __init__(self, input_path):
        print("Creating Page from file:", input_path)
        super().__init__(input_path)

    def __str__(self):
        return "Page at {}: {}".format(self.url, self.title)



# Starter configuration file copied on init

### local directory setup
DIR_PUBLISH   = "_pub"
DIR_POSTS     = "posts"
DIR_PAGES     = "pages"
DIR_DRAFTS    = "drafts"
DIR_STATIC    = "static"
DIR_TEMPLATES = "templates"
POST_STORE    = "post_store.db"

### public-facing site metadata
SITE = {
    'title':        "Stasis",
    'description':  "A simple static site generator with deployment to S3/Cloudfront",
    'site_name':    "example.com",
    'author':       ("author", "/about/"),
    'author_name':  "Namely McNameworthy",
    'absolute_url': "https://example.com",
    'base_url':     "",
    'twitter':      "@example",
    'github':       "https://github.com/example",
    'links': [
        ("Home", "/"),
        ("About", "/about/"),
        ("Wikipedia", "https://en.wikipedia.org/wiki/Main_Page"),
        ("RSS Feed", "/feed.xml"),
        ("README", "/readme/")
    ],
    'recent_posts': 10,
    'maincss':      "/css/main.css",
    'rssfeed':      "feed.xml"
}

PANDOC_ARGS = "markdown+backtick_code_blocks+inline_code_attributes"
EXCERPT_SEPARATOR = "<!--excerpt-->"
TOPICS_URL_PREFACE = "topics"

### pagination
POSTS_PER_PAGE = 10

### development server
SERVER_PORT = 9988

### deployment
AWS_PROFILE = "aws_profile_name"
S3_BUCKET = "s3_bucket_name"
S3_USE_GZIP = True
S3_GZIP_COMPRESSION = 9
S3_GZIP_FILES = [
    "*.html",
    "*.css",
    "*.js",
    "*.txt"
]
S3_GZIP_MINSIZE = 1024

### files matching these patterns will never be updated or deleted from S3
IGNORE_ON_SERVER = [
    "legacy*",
    "contact.html",
]

### files matching these patterns will bever be uploaded to S3
EXCLUDE_FROM_UPLOAD = [
    "*.DS_Store",
    "*.upload",
]

# Default format for post urls: "/YYYY/MM/DD/slug/"
def FN_POST_URL(args):
    url = "/{}/{}/{}/{}/".format(
        args['meta']['date'].strftime("%Y"),
        args['meta']['date'].strftime("%m"),
        args['meta']['date'].strftime("%d"),
        args['input_path'].stem[11:]
    )
    return url





import pathlib

import frontmatter

from .basepage import BasePage

class Post(BasePage):
    def __init__(self, input_path, is_draft=False):
        super().__init__(input_path)
        self.next_post = None
        self.prev_post = None
        self.is_draft = is_draft

    def get_url(self, meta, urlfn):
        args = {
            'meta': meta,
            'input_path': self.input_path
        }
        return urlfn(args)

    def __str__(self):
        if self.date:
            return "Post on {}: {}".format(self.date.isoformat(), self.title)
        else:
            return "Post: {}".format(self.title)




import http.server
import socketserver


# https://stackoverflow.com/questions/39801718/
def handler_from(directory):
    def _init(self, *args, **kwargs):
        return http.server.SimpleHTTPRequestHandler.__init__(self, *args, directory=self.directory, **kwargs)
    return type(f'HandlerFrom<{directory}>',
                (http.server.SimpleHTTPRequestHandler,),
                {'__init__': _init, 'directory': directory})


def serve(args, config):

    rootdir = config['DIR_PUBLISH']
    port = config['SERVER_PORT']
    httpd = socketserver.TCPServer(("localhost", port), handler_from(rootdir))
    print("Starting server at: http://localhost:{}/".format(port))
    try:
        httpd.serve_forever()
    except KeyboardInterrupt:
        pass

    print("Stopping server.")
    httpd.server_close()



import pathlib
import importlib

DEFAULT_CONFIG_FILE = "stasis_config.py"
BOOTSTRAP_PATH = "bootstrap"

def read_config():
    config_path = pathlib.Path(DEFAULT_CONFIG_FILE)
    if config_path.exists():
        global_vars = {}
        local_vars = {}
        with open(config_path.resolve()) as f:
            exec(f.read(), global_vars, local_vars)
        return local_vars
    else:
        return {}



import subprocess
import os
import pathlib


class Pandoc():
    def __init__(self, config):
        self.env = {
            **os.environ,
            "LINK_PREFIX": config['SITE']['base_url'],
        }
        self.filters_path = pathlib.Path(__file__).parent.resolve()
        self.fixlinks = "{}/filters/fixlinks.lua".format(str(self.filters_path))
        self.wordcount = "{}/filters/wordcount.lua".format(str(self.filters_path))
        self.args = config['PANDOC_ARGS']

    def markup(self, input):
        encoded_input = input.encode('utf-8')
        cmd = [
            "pandoc",
            "--from", self.args,
            "--to", "html",
            "--lua-filter={}".format(self.fixlinks)
        ]

        res = subprocess.run(cmd, input=encoded_input, capture_output=True, env=self.env)
        return res.stdout.decode('utf-8')

    def countwords(self, infile):
        lua_filter = "{}/filters/wordcount.lua".format(str(pathlib.Path(__file__).parent.resolve()))
        cmd = ["pandoc", "--lua-filter", lua_filter, infile]
        res = subprocess.run(cmd, stdout=subprocess.PIPE)
        wordcount = int(res.stdout.decode('utf-8'))
        return wordcount



import pathlib
import shutil

def clean(args, conf):

    print("Cleaning site output directory...")
    site_dir = pathlib.Path('.') / conf['DIR_PUBLISH']
    if site_dir.exists():
        shutil.rmtree(site_dir)
    site_dir.mkdir()
    post_store_path = pathlib.Path('.') / conf['POST_STORE']
    post_store_path.unlink(missing_ok=True)



from setuptools import setup, find_packages
import pathlib

here = pathlib.Path(__file__).parent.resolve()
long_description = (here / 'README.md').read_text(encoding='utf-8')
from stasis._version import __version__

setup(
    name='stasis-ssg',
    version=__version__,
    description='A simple static site generator with deployment to S3/Cloudfront',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/czep/stasis',
    author='Scott Czepiel',
    author_email='dev@czep.net',
    classifiers=[
        'Development Status :: 3 - Alpha',

        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Text Processing :: Markup :: Markdown',

        'License :: OSI Approved :: MIT License',

        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3 :: Only',
    ],
    keywords='static site generator, web development, blog',
    packages=find_packages(),
    python_requires='>=3.9, <4',
    install_requires=['jinja2', 'boto3', 'python-frontmatter'],

    package_data={
        'stasis': [
            'filters/*',
            'bootstrap/*'
        ],
    },
    include_package_data=True,

    entry_points={
        'console_scripts': [
            'stasis=stasis:main',
        ],
    },
)



import pathlib
import re
import datetime
from dateutil.tz import tzutc
import fnmatch
import mimetypes
import os
import pprint
import hashlib
import string
import gzip
import shutil

import boto3

# fixed timestamp to insert into gzip header
# required to ensure that subsequent gzips of the same
# file will have the same hash
# see: https://stackoverflow.com/questions/264224/
MAGICTIME = 999999999

def keysort(d):
    return {k: d[k] for k in sorted(d)}

def to_uri(path, basepath):
    return re.sub(basepath, '', str(path.resolve()))

def to_date(ts):
    return datetime.datetime.fromtimestamp(ts, tzutc())

def pattern_match(filename, pattern_list):
    found = False
    for pattern in pattern_list:
        if fnmatch.fnmatch(filename, pattern):
            found = True
            break
    return found

def get_object_dict(filelist):
    object_list = []
    for f in filelist:
        object_list.append({'Key': f})
    d = {'Objects': object_list}
    return d

def md5(file):
    md5hash = hashlib.md5()
    with open(file, "rb") as f:
        for chunk in iter(lambda: f.read(4096), b""):
            md5hash.update(chunk)
    return md5hash.hexdigest()

def files_differ(local, remote):
    if local['ETag'] != remote['ETag']:
        return True
    else:
        return False

def compress_copy(filename, compresslevel):
    zipfilename = filename.with_name(str(filename.name) + ".upload")
    with open(filename, 'rb') as f_in:
        with gzip.GzipFile(zipfilename, 'wb', compresslevel=compresslevel, mtime=MAGICTIME) as f_out:
            shutil.copyfileobj(f_in, f_out)
    return zipfilename


class S3():
    def __init__(self, args, config):
        self.s3_bucket = config['S3_BUCKET']
        self.aws_profile = config['AWS_PROFILE']
        self.local_dir = config['DIR_PUBLISH']
        self.local_dir_abs = str(pathlib.Path(config['DIR_PUBLISH']).resolve()) + '/'
        self.exclude_from_upload = config['EXCLUDE_FROM_UPLOAD']
        self.ignore_on_server = config['IGNORE_ON_SERVER']
        self.dry_run = args.dry_run
        self.puncts = re.compile('[%s]' % re.escape(string.punctuation))
        self.use_gzip = config['S3_USE_GZIP']
        self.gzip_compression_level = config['S3_GZIP_COMPRESSION']
        self.gzip_file_extensions = config['S3_GZIP_FILES']
        self.gzip_minsize = config['S3_GZIP_MINSIZE']
        self.cloudfront_id = None
        if 'CLOUDFRONT_ID' in os.environ:
            self.cloudfront_id = os.environ['CLOUDFRONT_ID']


    def deploy(self):
        print("Deploying site to S3 bucket: {}".format(self.s3_bucket))
        session = boto3.session.Session(profile_name=self.aws_profile)
        client = session.client('s3')

        local_files = self.get_local_files()
        remote_files = self.get_remote_files(client)
        newfiles = self.get_new_files(local_files, remote_files)
        updated = self.get_updated_files(local_files, remote_files)
        oldfiles = self.get_old_files(local_files, remote_files)

        print("\nUploading {} new files to S3...".format(str(len(newfiles))))
        for f in newfiles:
            self.upload_file(newfiles[f], client)

        print("\nModifying {} existing files on S3...".format(str(len(updated))))
        for f in updated:
            self.upload_file(updated[f], client)

        print("\nDeleting {} old files from S3...".format(str(len(oldfiles))))
        for f in oldfiles:
            self.delete_from_s3(oldfiles, client)

        remote_actions = len(newfiles) + len(updated) + len(oldfiles)
        if remote_actions > 0 and not self.dry_run and self.cloudfront_id is not None:
            self.invalidate_cloudfront(session)


    def get_local_files(self):
        paths = pathlib.Path(self.local_dir).glob('**/*')
        files = {}
        for p in paths:
            if p.is_file():
                key = to_uri(p, self.local_dir_abs)
                val = {
                    'localpath': p,
                    'LastModified': to_date(p.stat().st_mtime),
                    'Size': p.stat().st_size,
                    'ETag': md5(p)
                }
                files[key] = val
        return keysort(files)


    def get_remote_files(self, client):
        paginator = client.get_paginator('list_objects_v2')
        pages = paginator.paginate(Bucket=self.s3_bucket)
        files = {}
        for page in pages:
            if 'Contents' in page:
                for obj in page['Contents']:
                    objdict = {
                        'LastModified': obj['LastModified'],
                        'Size': obj['Size'],
                        'ETag': self.remove_puncts(obj['ETag'])
                    }
                    files[obj['Key']] = objdict
        return keysort(files)


    def get_new_files(self, local, remote):
        newfiles = {}
        for f in local:
            if f not in remote and not pattern_match(f, self.exclude_from_upload):
                newfile = {
                    'key': f,
                    'localpath': local[f]['localpath']
                }
                if self.needs_gzip(local[f]):
                    zippedpath = compress_copy(local[f]['localpath'], self.gzip_compression_level)
                    newfile['zippedpath'] = zippedpath
                newfiles[f] = newfile
        return newfiles


    def needs_gzip(self, localfile):
        if not self.use_gzip:
            return False

        with open(localfile['localpath'], 'rb') as test_f:
            already_gzipped = test_f.read(2) == b'\x1f\x8b'

        if already_gzipped:
            return False

        if int(localfile['Size']) >= self.gzip_minsize and pattern_match(localfile['localpath'].name, self.gzip_file_extensions):
            return True
        else:
            return False


    def get_updated_files(self, local, remote):
        updated = {}
        for f in local:
            if f in remote and not pattern_match(f, self.ignore_on_server):
                is_modified = False
                zippedpath = None
                if self.needs_gzip(local[f]):
                    zippedpath = compress_copy(local[f]['localpath'], self.gzip_compression_level)
                    if md5(zippedpath) != remote[f]['ETag']:
                        is_modified = True
                else:
                    if local[f]['ETag'] != remote[f]['ETag']:
                        is_modified = True
                if is_modified:
                    updated[f] = {
                        'key': f,
                        'localpath': local[f]['localpath']
                    }
                    if zippedpath is not None:
                        updated[f]['zippedpath'] = zippedpath
        return updated


    def get_old_files(self, local, remote):
        oldfiles = []
        for f in remote:
            if f not in local and not pattern_match(f, self.ignore_on_server):
                oldfiles.append(f)
        return oldfiles


    def upload_file(self, file, client):
        local_filename = file['localpath'].resolve()
        remote_key = file['key']
        mimetype, _ = mimetypes.guess_type(local_filename)
        if mimetype is None:
            mimetype = "application/octet-stream"
        metadata = {
            "ContentType": mimetype
        }
        upload_filename = local_filename
        if 'zippedpath' in file:
            upload_filename = file['zippedpath'].resolve()
            metadata['ContentEncoding'] = "gzip"

        print("{}\t{}".format(remote_key, metadata))

        if not self.dry_run:
            client.upload_file(
                Filename=str(upload_filename),
                Bucket=self.s3_bucket,
                Key=remote_key,
                ExtraArgs=metadata
            )


    def delete_from_s3(self, filelist, client):
        if len(filelist) == 0:
            return
        deletedict = get_object_dict(filelist)
        for f in filelist:
            print(f)
        if not self.dry_run:
            client.delete_objects(Bucket=self.s3_bucket, Delete=deletedict)


    def invalidate_cloudfront(self, session):
        client = session.client('cloudfront')
        ts = datetime.datetime.utcnow()
        print("Creating Cloudfront invalidation with caller reference: ", str(ts))
        response = client.create_invalidation(
            DistributionId=self.cloudfront_id,
            InvalidationBatch={
                'Paths': {
                    'Quantity': 1,
                    'Items': [
                        '/*'
                    ],
                },
                'CallerReference': str(ts)
            }
        )
        pprint.pprint(response)


    def remove_puncts(self, s):
        return self.puncts.sub('', s)




import pathlib
import shutil
import importlib.resources
from distutils.dir_util import copy_tree

from .config import *

def init(args, conf):

    print("Initializing new static site...")
    print("Target directory: {}".format(pathlib.Path('.').resolve()))

    # do not continue unless current directory is empty
    if any(pathlib.Path('.').iterdir()):
        print("Ensure target directory is empty before initializing!")
        return

    bootstrap_path = pathlib.Path(__file__).parent / BOOTSTRAP_PATH
    copy_tree(bootstrap_path.resolve(), str(pathlib.Path('.').resolve()))




import pathlib
import datetime

import frontmatter

class BasePage():
    def __init__(self, input_path):
        self.input_path = input_path
        self.output_path = None
        self.layout = None
        self.url = None
        self.title = ""
        self.date = None
        self.wordcount = 0
        self.topics = []
        self.content_text = None
        self.content_html = None
        self.excerpt_text = None
        self.excerpt_html = None

    def build(self, pandoc, config):
        with self.input_path.open() as infile:
            self.content_text = infile.read()
        meta = frontmatter.loads(self.content_text)
        self.layout = meta['layout']
        self.title = meta['title']
        self.date = self.get_date(meta, self.input_path)
        self.url = self.get_url(meta, config['FN_POST_URL'])
        self.output_path = self.get_output_path(config['DIR_PUBLISH'])
        self.make_excerpt(config['EXCERPT_SEPARATOR'])
        if not self.is_current():
            self.generate_html(pandoc)
        self.wordcount = pandoc.countwords(self.input_path)
        self.topics = ('topics' in meta and meta['topics'].split(" ")) or []

    def get_date(self, meta, input_path):
        pagedate = None
        try:
            pagedate = datetime.date.fromisoformat(str(meta['date']))
        except:
            pass
        if pagedate is None:
            try:
                pagedate = datetime.date.fromisoformat(str(self.input_path.stem[:10]))
            except:
                pass
        if pagedate is None:
            pagedate = datetime.date.today()
        return pagedate

    def get_url(self, meta, urlfn):
        return meta['path']

    def get_output_path(self, site_path):
        path = self.url
        if path.startswith("/"):
            path = path[1:]
        if path.endswith("/"):
            path += "index.html"
        return site_path / pathlib.Path(path)

    def make_excerpt(self, excerpt_sep):
        splittext = self.content_text.split(excerpt_sep, maxsplit=1)
        if len(splittext) == 2:
            self.excerpt_text = splittext[0]
            self.content_text = splittext[1]

    def generate_html(self, pandoc):
        self.content_html = pandoc.markup(input=self.content_text)
        if self.excerpt_text is not None:
            self.excerpt_html = pandoc.markup(input=self.excerpt_text)

    def is_current(self):
        if self.output_path is None or not self.output_path.exists():
            return False
        if self.input_path.stat().st_mtime >= self.output_path.stat().st_mtime:
            return False
        else:
            return True

    def write(self, jinja_env, site_meta):
        if not self.is_current():
            print("Writing:", self)
            template = jinja_env.get_template("{}.html".format(self.layout))
            page = self.get_page_metadata()
            template_out = template.render(content=self.content_html, page=page, post=self, site=site_meta)
            self.output_path.parent.mkdir(parents=True, exist_ok=True)
            self.output_path.write_text(template_out)

    def get_page_metadata(self):
        page = {
            'title': self.title,
            'excerpt': (hasattr(self, 'excerpt_html') and self.excerpt_html) or '',
            'url': self.url
        }
        return page



from .s3 import S3

def deploy(args, conf):
    msg_suffix = ""
    if args.dry_run:
        msg_suffix = "  DRY RUN"
    print("Deploying site...", msg_suffix)
    s3 = S3(args, conf)
    s3.deploy()

